### ----------------------------------------------------------------------
#
# The user visible interfaces to drag-and-drop
#

# Example (very stupid) drag handler
proc accepts_drop args {return 1}

# Example generator for strings (for use see demo)
proc supplies_string {w s} {
    global dragdrop

    set dragdrop(supplies,$w) $s
    return STRING
}
# Example dropped data server
proc supply_string {w t} {
    global dragdrop

    if {$t != "STRING"} {
	return -code error "Wrong type code: $t instead of STRING"
    }
    return $dragdrop(supplies,$w)
}

# Sets up widget as a drag-n-drop source
#
# Args are:
#   widget    - What widget should get these abilities?
#               NB. Will probably conflict with other bindings
#   server    - Server to call to get dropped data. Takes
#               {widget name} and {data type} as args
#   generator - Command to run at start of drag operation.
#               Has value of types of data supplied
#   button    - Button to bind
#   notifier  - Server to inform app of current drag-n-drop state.
proc dragdrop_bind {widget server generator button {notifier {}}} {
    global dragserver dragdropnotify

    bind $widget <Button-$button>        "drag_start %X %Y \[$generator\]"
    bind $widget <B$button-Motion>       {drag %X %Y}
    bind $widget <ButtonRelease-$button> {drop %X %Y}
    set dragserver($widget)     $server
    set dragdropnotify($widget) $notifier
}

# Sets up widget as a drag-n-drop destination
#
# Args are:
#   widget           - What widget should get these abilites?
#   dragging_handler - Server to call to find if the widget wants to accept
#                      data at that position, and is also used to implement
#                      look-ahead positioning. Called with all args empty
#                      when the mouse leaves the widget.
#   types            - Types of drop data accepted, _in_preferred_order_.
#   dropping_handler - Server to call to handle actual drop of data.
proc dragdrop_destination {widget dragging_handler types dropping_handler} {
    global draghandler drophandler dropacceptedtypes

    set draghandler($widget)       $dragging_handler
    set drophandler($widget)       $dropping_handler
    set dropacceptedtypes($widget) $types
}
